#A series of functions to turn gas adsorption data from a BELSORP-HP device in HPD format into isotherm plots.

import os, time
import matplotlib.pyplot as plt
'''
ident, column number
Adsorptoion, 0
STEP No., 1
Pi (kPa), 2
Pe (kPa), 3
Pe2 (kPa), 4
P0 (kPa), 5
Ti (°C),6
Te (°C), 7
Te2 (°C), 8
Meas. Temp.(K), 9
Vs(cm3), 10
Ads. amount (cc(STP)/g), 11
'''
defaultkwargs = {'title':'', 'gtype':'cm3/g', 'des' : 'yes', 'maxp': 9999, 'ltype':'gastemp',
'cols': "green,red,blue,orange,yellow", 'shaps':'s,D,o,^,p'}

folder = os.getcwd() +'\\' #"C:\\Users\\kcj\\Desktop\\gas sorp zncanet4n\\dd_test\\" #
file_idents = [i[:-4] for i in os.listdir(folder) if i.endswith('.HPD')]

def sampleID(ident_num, ident_start):
    filepath = folder + ident_num + '.HPD'
    fileobj = open(filepath, 'r')
    for line in fileobj:
        if line.startswith(ident_start):
            return line[len(ident_start):]

def returndata(ident_num, column_num):
    filepath = folder + ident_num + '.HPD'
    fileobj = open(filepath, 'r')
    temp_data = []
    for line in fileobj:
        if len(line) > 80:
            temp_data.append(line.split(',')[column_num])
    return temp_data

def ads_get(ident_num):
    return sampleID(ident_num, ',Adsorptive,')[:-1]
    
def temp_get(ident_num):
    return sampleID(ident_num, ',Adsorption temperature,')[:-5] + 'K'

def get_density(ident_num): # need a source for these
    ads = sampleID(ident_num, ',Comment3,')[:3]
    if ads == 'CO2': return 44.013/22.415
    if ads == 'CH4': return 16.04/22.415
    if ads == 'H2 ': return 2.016/22.415
    if ads == 'N2 ': return 28.013/22.415
    if ads == 'N2O': return 44.013/22.415
    if ads == 'N20': return 44.013/22.415
    else:
        print "gas not found, numbers unreliable"
        print ads
        return 1.0

def chemtex(str):
    return "$\mathsf{" + ''.join(["_"+i if i.isdigit() else i for i in str])+"}$"

def desorption_split(ident_num): 
    try:
        return returndata(ident_num, 1)[2:].index('1')+2
    except ValueError:
        return len(returndata(ident_num, 1)[2:])
        
def multi_isotherm_graph(*idents, **kwargs):
    fig = plt.figure()
    ax = fig.add_subplot(111)
    kwargs = dict(defaultkwargs.items() + kwargs.items())
    cols = kwargs['cols'].split(',')
    shaps = kwargs['shaps'].split(',')
    for index,ident_num in enumerate(idents):
		y = returndata(ident_num, 11)[1:]
		y = [i for j,i in enumerate(y) if float(i)>0]
		x = returndata(ident_num, 4)[1:][:len(y)]
		de = desorption_split(ident_num)-1
		density = get_density(ident_num)
		molar_mass = density * 22.415
		ads = ads_get(ident_num)
		temp = temp_get(ident_num)
		maxp = float(kwargs['maxp'])
		if maxp < max([float(i) for i in x]):
			x = [i for j,i in enumerate(x) if float(i)<maxp and j<de]
			y = y[:len(x)]
		if kwargs['gtype'] == 'g/g':
			y2 = [float(i)*density/1000.0 for i in y]
			ax.set_ylabel(chemtex(ads+'\ uptake\ (g/g)'), size = 'x-large')
			for i in range(len(y2)):
				if y2[i]==max(y2):
					print y2[i], x[i], temp
		elif kwargs['gtype'] == 'cm3/g':
			y2 = y
			ax.set_ylabel(chemtex(ads)+'$\mathsf{\ uptake\ (cm^3\ (STP)/g)}$', size = 'x-large')
		elif kwargs['gtype'] == 'mmol/g':
			y2 = [float(i)/22.415 for i in y]
			ax.set_ylabel('$\mathsf{\ uptake\ (mmol\ g^{-1})}$', size = 'x-large')
		else:
			y2 = y
			ax.set_ylabel(chemtex(ads)+'$\mathsf{\ uptake\ (cm^3\ (STP)/g)}$', size = 'x-large')
		if kwargs['ltype'] == 'gastemp':
			label2 = chemtex(ads) + ' $\mathsf{'+ temp +'}$'
		elif kwargs['ltype'] == 'comp':
			label2 = ident_num.split('_')[0]
		else: print 'ltype not recognised'
		if kwargs['des'] == 'no':
			print 'no desorption'
		else: 
			ax.plot(x[de-1:], y2[de-1:],shaps[index], linestyle = 'solid', lw = 0.5,
			color = 'grey', markerfacecolor='white', markersize=6)
		ax.plot(x[:de], y2[:de], shaps[index], linestyle = 'solid', lw = 0.5,
		color = 'black', markerfacecolor=cols[index], markersize=8,
		label = label2)
		ax.set_title(kwargs['title'] + '\n', size = 'x-large')
		ax.set_xlabel('$\mathsf{P\ (kPa)}$', size = 'x-large')
    ax.legend(loc = 4, numpoints = 1)
    #fig.show()
    title_alnum = ''.join(e for e in kwargs['title'] if e.isalnum())
    #fig.savefig(folder + title_alnum +".png", transparent = True, dpi = 1000)
    fig.savefig(folder + str(time.time()) +".png", transparent = True, dpi = 1000)
    
#multi_isotherm_graph('20161110','20161115','20161116',# title = '6,3-', 
#gtype = 'cm3/g', des = 'yes', maxp = 9999, ltype = 'gastemp')

#multi_isotherm_graph('20161121','20161123', gtype = 'g/g')
#multi_isotherm_graph('20161121')
'''
multi_isotherm_graph('Zn hba_N2O_298K','Zn hba_N2O_298K',
'Zn N2O_298K','Zn N2O_298K',
title = 'Substituted networks $\mathsf{N_2O}$ sorption', 
#gtype = 'cm3/g', des = 'yes', maxp = 2000, ltype = 'comp',
cols = 'r,g,b,y', shaps = 's,s,s,s'
)

type data points from highest to lowest on the legend. data will tend to run over
prevoius, so go from high T to low T

kwargs are optional. will default to values at top of page.

title = self explanatory. can use latex. in quotes. default none.
gtype = 'cm3/g' or 'g/g'. default cm3/g
des   = 'yes' or 'no'. default yes
maxp  = number, no quotes, in kpa. default 9999
ltype = 'gastemp' (i.e. N2 77K) or 'comp' (from filename, all before underscore). default gastemp
cols  = list of csv colours (i.e. 'red, green, blue' or 'r,g,b,y'). see http://matplotlib.org/api/colors_api.html
shap  = list of shapes (i.e. 's,D,o,^,p'). see http://matplotlib.org/api/markers_api.html
'''